#!/bin/bash
#garmin_download_0.412.sh
#------------------------------------------------------------------------------
#   Dave's GPS bash script is used to upload & download tracklogs, waypoints &
#   routes Garmin eTrex GPS and saves them to date-named ".gpx" & "csv" files.
#   The script also manages the Garmin eTrex GPS and runs TangoGPS.  It has
#   only been tested to run in bash and fails if run with 'sh'
#
#   COPYRIGHT:
#   Yeah Right! - (do what you like with it) but ...
#   This code comes with ABSOLUTELY NO WARRANTY.
#   It is open source and is released under the GPL
#   See: http://www.gnu.org/licenses/gpl.txt for full details.
#
#   IDEAS & COMMENTS:
#   This script was developed form ideas gleaned from many sources,
#   I don't claim to be any sort of guru and I would welcome any
#   sugestions, comments or improvements to: dave at 9gerkin9@gmail.com
#
#   TESTED ON:
#   The script has been used successfully on EeePC with the following OS's
#   1.  Ubuntu 8.10 Linux, kernel 2.6.24-21-eeepc
#   2.  CrunchBang 8.10.02 Linux, kernel 2.6.27-8-eeepc-lean
#   3.  CrunchBang 9.04.01 Linux, kernel 2.6.28-16-generic
#
#   & Desktop PC with the following OS's
#   1.  Ubuntu 8.10 Linux desktop edition, kernel 2.6.28-11-generic 
#   2.  Ubuntu 9.04 Linux desktop edition, kernel 2.6.28-13-generic
#   3.  Ubuntu 9.04 Linux desktop edition, kernel 2.6.28-15-generic
#   4.  Ubuntu 9.10 Linux desktop edition, kernel 2.6.31-14-generic
#
#   HARDWARE:
#   Garmin eTrex (yellow basic model), firmware version 3.60 &
#   Garmin eTrex H, firmware version 3.10 using a
#   standard generic serial connection & also, Prolific Technology, Inc.
#   PL2303 based chipset USB to serial DB9 converter
#
#   DEPENDENCIES:
#   1. This script uses latest Gpsbabel - version 1.3.6, which now includes support
#      for the Holux M-241.  It was compiled from source and available from:
#        http://www.gpsbabel.org/downloaded.html
#   2. Perl script "convert2gpx.pl" to convert TangoGPS log files 
#      avaliable from:  http://www.tangogps.org/downloads/convert2gpx.pl
#
#   CREDITS:
#
#   For all this goodness see:
#   http://www.gpsbabel.org/index.html - converts waypoints, tracks, and routes
#      between popular GPS receivers and mapping programs
#   http://www.rigacci.org/ - for MTKBabel info & available from:
#   http://kambing.ui.edu/debian/pool/main/m/mtkbabel/ -MTKBabel is a Perl program 
#      to operate GPS devices based on the MediaTek MTK chipset
#   http://gpsvisualizer.com/ - a fantastic web based maps, GPS converters etc.
#   http://www.tangogps.org/gps/cat/About - TangoGPS is an easy to use, fast and
#      lightweight mapping application for use with or without GPS
#   http://openstreetmap.org/ - OpenStreetMap is a free editable map of the whole world
#   http://www.ubuntu.com/ - community developed, linux-based operating system
#
#   Thanks to CLUG N.Z members for their suggestions & Gav at: http://www.rediguana.co.nz/gav/
#   with help on figuring out how to use the gpsbable filters.
#
filename=$HOME/bin/garmin_download_0.412.sh
version="Verson 0.412 - testing - 23 Jan 2010"
#
#==============================================================================
#==================	  SET YOUR USER VARIBLES IN THIS SECTION	===============
#==============================================================================
#   List the GPS device:
device="Garmin eTrex H"

#   Local folders where files are stored:
#  (the default locations here probably wont need to be changed)

gps=$HOME/GPS
gpc=$HOME/GPS/geocache
gpo=$HOME/GPS/osm
gpi=$HOME/GPS/print
gpr=$HOME/GPS/routes
gpt=$HOME/GPS/temp
gpk=$HOME/GPS/tracks
gpu=$HOME/GPS/upload
gpw=$HOME/GPS/waypoints
gpp=$HOME/GPS/WP
bin=$HOME/bin

#   Set default USB download port:
usbport=/dev/ttyUSB0	# (default: /dev/ttyUSB0)

#   Set default serial download port:
srlport=/dev/ttyS0		# (default: /dev/ttyS0)

#   Specify the default waypoint upload file:
# eg. "mycurrentwaypoints.gpx" (and put file in the ~/GPS/waypoints directory)
dwpt=waygar-090424.gpx

#   Specify the default geocache upload file:
# eg. "mycurrentgeocaches.gpx" (and put file in the ~/GPS/geocache directory)
dgeo=geocache-0905312316.gpx

#   Set radius distance (eg. 0.5K, 1M) and lat/lon cordinates
#   (eg. somewhere near home location to filter tracklogs for OpenStreetMap):
rdllc="distance=1K,lat=-43.580,lon=172.566"

#   Specify the TangoGPS log file convert script:
tangcv=$bin/convert2gpx.pl

#   Specify the print waypoints convert script:
prtwpt=$bin/print_waypoints_0.18.sh
#TESTING#prtwpt=$HOME/print_waypoints_0.17.sh

#   Specify the polygon filter file:
poly=$HOME/bin/polygon_south.txt	# (not active in script yet)

#   set the default location TangoGPS saves the log file when in "Tracks (3/4)" here:
# (this is the location set in the "Tracks > Configure > Trackdir Config" setting)
tango=$HOME/GPS/temp

#   Global script variables (used in multiple sections):
cvsfile=trkgar-`date +%y%m%d-%H%M`.csv  # TangoGPS timestamped trackfile name
trkfile=trkgar-`date +%y%m%d-%H%M`.gpx  # Garmin timestamped trackfile name
wptfile=waygar-`date +%y%m%d-%H%M`.gpx  # Garmin timestamped waypoints filename
rtefile=rtegar-`date +%y%m%d-%H%M`.gpx  # Garmin timestamped routes filename
gchfile=gcache-`date +%y%m%d-%H%M`.gpx  # time-stamped geocaching.gpx file

#==============================================================================
#===================	DON'T EDIT BELOW THIS LINE		=======================
#==============================================================================

echo -e "\n     ====================================================================\n\
\t\t    Dave's $device script\n\t\t      $version\n\n\t    This script uploads & downloads: tracklogs, waypoints &\n\t     routes from the $device - using GPSBabel & GPSD\n     ====================================================================\n"
{
#   check for the necessary GPS directory(s) & if not create them:
echo -e "\tChecking for required dependencies:"; sleep 1
if [ ! -d $gpw ]; then
    echo -e "\n\t> > > Required GPS download directories missing in $HOME < < <\n"
    read -p "        ... shall I create the download directories now? (Y/N) > "
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then 
	echo -e "\t... please specify download directories and run again... exiting\n"; exit 0
    fi
    if [ "$REPLY" = "n" ] ; then 
	echo -e "\t... please specify download directories and run again... exiting\n"; exit 0
    fi
    if [ "$GO" = "1" ]; then
	echo -e "\t... creating the required directories"
	directories="$gps $gpc $gpo $gpi $gpr $gpt $gpk $gpu $gpw $gpp $bin"
	for d in $directories
	do
		if test -d ${d}
		then
		    echo >> /dev/null
		else 
			echo -e "\t${d} not found ..."; sleep 1
			mkdir ${d}
			echo -e "\t${d} created"
		fi
	done
	fi
else
	echo -e "\t... GPS download directories are present"
fi    
}
#   check for required programs needed to run this script:
#   (if a required program is not present then print error and exit)
gpsbabel=`which gpsbabel`
#   test to see if GPSBabel is installed:
if [ -z $gpsbabel ]; then
    echo -e "\n\t> > > Cannot find the required program \"gpsbabel\" < < <\n\n\t... version 1.3.6 can be found at:\n\t    http://www.gspbabel.org, or install it with:\n\t    \"sudo apt-get install gpsbabel\" and run again\n"; exit 0
#   test to see if  the "convert2gpx.pl" script is present:
elif [ ! -e "$tangcv" ]; then
    echo -e "\n\t> > > Cannot find the program \"`basename $tangcv`\" < < <\n\n\t... The latest version can be found at:\n\t    http://www.tangogps.org/downloads/convert2gpx.pl\n\n\t    \"`basename $filename`\": \n\t    will still work but will have reduced functionality\n"
#   test to see if  the "print_waypoints_0.x.shl" script is present:
#   (comment out the next 2 lines if this functionality isn't reqd.)
elif [ ! -e "$prtwpt" ]; then
    echo -e "\n\t> > > Cannot find the program \"`basename $prtwpt`\" < < <\n\n\t... The latest version can be found at:\n\t    http://dl.getdropbox.com/u/1448458/gps_download.tar.gz\n\n\t    \"`basename $filename`\": \n\t    will still work but will have reduced functionality\n"
else
echo -e "\t... all required programs are present\n"
#fi; echo -e "\t... all required programs are present\n"
fi
#=================================section break================================
## SET THE COMM PORT ##
echo -e "\n\t--------------------------------\n\t CONNECT OPTIONS\n\t--------------------------------\n\n\tSelect connection for the $device\n"

#   Select connection:
choice_of()
select opt
do
   echo -e "\t$opt selected\n"
   break
done

#   Define options for this section:
choice_of "USB connection" "Serial connection" "Next menu" "Quit"

#------------------------------------option------------------------------------
#   Option 1 - Connect via USB port:
if [ "$opt" = "USB connection" ]; then
    echo -e "\tCheck the $device is connected to USB & turned on\n"; port=$usbport
    
#------------------------------------option------------------------------------
#   Option 2 - Connect via Serial connection:
elif [ "$opt" = "Serial connection" ]; then
    echo -e "\tCheck the $device is connected to serial & turned on\n"; port=$srlport
    
#------------------------------------option------------------------------------
#   Option 3 - Jump to the next section:
elif [ "$opt" = "Next menu" ]; then
	echo

#------------------------------------option------------------------------------
#   Option 4 - Quit this script:
elif [ "$opt" = "Quit" ]; then
    echo -e "\t... exiting\n"; exit 0
else
    clear; echo -e "\t... bad option\n"; exit 0
fi

#=================================section break================================
## DOWNLOAD TRACKLOG DATA ##
echo -e "\n\t--------------------------------\n\t\tDOWNLOAD OPTIONS\n\t--------------------------------\n\n\tSelect download options for the $device\n"

#   Select download options:
choice_of()
select opt
do
   echo -e "\t$opt selected\n"
   break
done

#   Define options for this section:
choice_of "Download the tracklog from the $device" "Download the current position" "Download all waypoints from the $device" "Download all routes from the $device" "Next menu" "Quit"

#------------------------------------option------------------------------------
#   Option 1 - Download the tracklog data:
if [ "$opt" = "Download the tracklog from the $device" ]; then
   echo -e "\t... downloading tracklogs"

#   Download the tracklog from the GPS & renames it to the correct directory:
	sleep 10 && $gpsbabel -t -i garmin -f $port -o gpx -F $gpt/$trkfile &&
if [ ! -e "$gpt/$trkfile" ]; then
    clear; echo -e "\n\t\t> > > Transfer failed < < <\n\tCheck the following:\n\t a. the $device connected & turned on\n\t b. the correct \"port\" is specified\n\t c. no virtual machines are running (VMs can lock the output port)\n\t... exiting\n"; exit 0
else
    echo -e "\t... track downloaded and saved as: ~/GPS/temp/\"$trkfile\"\n\t    (the filename - location will need to be edited by hand)\n\t    & the tracklog cleared manually from the $device\n"
fi
#########################  STILL TO ENABLE POLYGON FILTER ########################
#	The polygon filter is not enabled in the script however the radius filter
#	is active as listed below:
#
#   Filter data using ggsbabel to clean up the downloaded tracklog data:
#       a. create waypoints from tracks
#       b. remove all tracks
#       c. radius filter around home location (set radius distance & co-ords. below)
#       d. removed duplicates by location & shortname
#       e. create tracks from waypoints
#       f. remove all waypoints
#       g. appended to one another to form a single track
##################################################################################
$gpsbabel -t -i gpx -f $gpt/$trkfile -x transform,wpt=trk -x nuketypes,tracks -x radius,$rdllc,exclude,nosort -x transform,trk=wpt -x nuketypes,waypoints -x track,pack,split=30m,title=trkgar -o gpx -F $gpt/garmin.gpx &&

#   Change timestamp to 00:00 UTC:
sed 's/T[0-9][0-9]:[0-9][0-9]:[0-9][0-9]Z/T00:00:00Z/g' $gpt/garmin.gpx > $gpt/garmin-clean.gpx &&

#   Copy cleaned tracklog data to /temp directory:
$gpsbabel -i gpx -f $gpt/garmin-clean.gpx -o gpx -F $gpt/trkcln-`date +%y%m%d-%H%M`.gpx &&
{
#   Find the newest tracklog file in the ~/GPS/temp directory:
newcln=`ls -tr $gpt/trkcln-*.gpx | tail -1`
create=`ls -tr $gpt/trkgar-*.gpx | tail -1`
osmfile=osmgar-`date +%y%m%d-%H%M`.gpx  # OpenStreetMap road track file name
osmwpt=osmwpt-`date +%y%m%d-%H%M`.gpx  # OpenStreetMap road track file name
#   Check for the tracklog file in the ~/GPS/tracks directory:
if [ ! -e "$newcln" ]; then
###################   INSERT SCRIPT TO FIX TIME ERRORS HERE   ####################
#
#   Sometimes the Garmin produces a tracklog with time errors and GPSBabel 
#   is unable to successfully produce a cleaned tracklog ("trkcln-*.gpx")
#   This section will manually create an "osmfile" but it will probably
#   need editing by hand to remove any stray waypoints
#
##################################################################################
    echo -e "\t> > >  Cannot find the \"trkcln-*.gpx\" file in \"$gpt\" < < <\n\t... will try and create it now"; sleep 1 && cp $create $gpo/$osmfile && echo -e "\t    success, but the resulting file will need editing by hand"
else
    read -p "	Is this track going to be uploaded to OpenStreetMap? (Y/N)  "
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then
   	echo
    fi
    if [ "$REPLY" = "n" ] ; then 
   	echo
    fi
    if [ "$GO" = "1" ]; then
    # copy track and download waypoints to the OpenStreetMap directory:
	cp $newcln $gpo/$osmfile; $gpsbabel -w -i garmin -f $port -o gpx -F $gpo/$osmwpt &&
	echo -e "\t... OSM track and waypoints saved to: \"~/GPS/osm\"\n"
    fi
    #   Cleaning up - remove the temp data files & report completion:
    if [ ! -f "$newcln" ]; then
	    echo
    else
	    rm $newcln; rm -f $gpt/garmin.gpx; rm -f $gpt/garmin-clean.gpx && echo -e "\tFinished\n"
    fi
fi
}
#------------------------------------option------------------------------------
#   Option 2 - Grab current position & convert to waypoint:
elif [ "$opt" = "Download the current position" ]; then
	echo -e "\t... downloading current postion"
{
#   Position file names:
wptf=WP`date +%y%m%d-%H%M`.gpx
wptn=WP`date +%H%M`
wpty=current_position.gpx

#   Gpsbabel downloads the current latitude & longtitude and returns it as a single waypoint:
	sleep 10 && $gpsbabel -i garmin,get_posn -f $port -o gpx -F $gpt/$trkfile
if [ ! -e "$gpt/$trkfile" ]; then
    echo -e "\n\t> > > Tracklog failed to download < < <\n\tIs the $device connected & turned on?\n"; exit 0
else
    #   Convert output to waypoint file:
    $gpsbabel -i gpx -f $gpt/$trkfile -x transform,wpt=trk -o gpx -F $gpp/position.gpx
    #   Rename the <name>, <cmt> & <desc>  fields with timestamps/comments etc:
    sed 's/<name>[Position]*/<name>'$wptn'/' $gpp/position.gpx | 
    sed 's/<cmt>[Position]*/<cmt>Position as at: '`date +%H%M`' hrs '`date +%d.%m.%Y`'/' | 
    sed 's/<desc>[Position]*/<desc>'$wptn'/' > $gpp/$wptf 
fi

#   Run gpsbabel loop to create a waypoint "blop" that can be viewed with TangoGPS
#   for TangoGPS, load with: TangoGPS > Track > Load > "current_position.csv"
echo -e "\t... current position saved as:\n\t     1. ~/GPS/tracks/current_position.csv"
rm -f $tango/current_position.csv &&    #remove previous position point
for b in 1 2 3 4 5 6 7 8 9
do      # Beginning of loop
	$gpsbabel -i garmin,get_posn -f $port -o csv -F $gpt/$cvsfile &&
	tail $gpt/$cvsfile >> $tango/current_position.csv
	let "inner+=1"  # Increment loop counter
done	# End of of loop

#   Copy position files to folders and cleaning up - remove the temp data files & report completion:
cp $gpp/$wptf $gps/$wpty && echo -e "\t     2. ~/GPS/$wpty\n\t     3. ~/GPS/WP/$wptf\n"
rm -f $gpp/position.*; rm -f $gpt/$trkfile; rm -f $gpt/$cvsfile	&& echo -e "\tFinished\n\n"	# report completion
}	
#------------------------------------option------------------------------------
#   Option 3 - Download the waypoint data:
elif [ "$opt" = "Download all waypoints from the $device" ]; then
    echo -e "\t... downloading all waypoints"

##   Download the waypoints from the GPS & renames it to the correct directory:
	sleep 10 && $gpsbabel -w -i garmin -f $port -o gpx -F $gpt/$wptfile
if [ ! -e "$gpt/$wptfile" ]; then
    clear; echo -e "\n\t\t> > > Transfer failed < < <\n\tCheck the following:\n\t a. the $device connected & turned on\n\t b. the correct \"port\" is specified\n\t c. no virtual machines are running (VMs can lock the output port)\n\t... exiting\n"; exit 0
else
    echo -e "\t... downloaded and saved as: ~/GPS/temp/\"$wptfile\"\n\t    (filename will need to be edited by hand and copied to\n\t    ~/GPS/waypoints/ if appropriate) & the $device cleared\n\t    Clear the $device with the \"DELETE ALL\" option\n\t    and select the \"Upload default waypoints\" option below\n\n\tFinished\n" 
fi	# report completion

#------------------------------------option------------------------------------
#   Option 4 - Download the route data:
elif [ "$opt" = "Download all routes from the $device" ]; then
    echo -e "\t... downloading all routes"

#	Download the tracklogs from the GPS & rename to the correct directory:
	sleep 10 && $gpsbabel -r -i garmin -f $port -o gpx -F $gpt/$rtefile
if [ ! -e "$gpt/$rtefile" ]; then
    clear; echo -e "\n\t\t> > > Transfer failed < < <\n\tCheck the following:\n\t a. the $device connected & turned on\n\t b. the correct \"port\" is specified\n\t c. no virtual machines are running (VMs can lock the output port)\n\t... exiting\n"; exit 0
else
	cd $gpt && route=`ls -tr r*e*.gpx | tail -1` && cd
	newrte=$gpt/$route
#   Check for route file in the temp directory:
    if [ ! -e "$newrte" ]; then
        echo -e "\n\t> > > No GPS route downloaded < < <"; exit 0
    else
		gwpt=`grep "rtept" $newrte -c`									# number of routepoints
#		echo -e "\t... \"$route\" downloaded from $device"				# route file name
	if [ $gwpt = 0 ]; then
#	Downloaded route contains no data:
		echo -e "\n\t< < < \"$route\": contains no turnpoints > > >\n"; exit 0
	else
        gnme=`head -12 $newrte | grep "<name>" | cut -d ">" -f 2 | cut -d "<" -f 1`	# short name
		gdsc=`head -12 $newrte | grep "<desc>" | cut -d ">" -f 2 | cut -d "<" -f 1` # long name
		if [ -n "$gnme" ]; then
			echo -e "\t... route: \"$gnme\"-$gwpt points, downloaded from $device"	# route name
		elif [ -n "$gdsc" ]; then
			echo -e "\t    description: \"$gdsc\"" 						# route description
		elif [ -n "$gwpt" ]; then
			echo -e "\t    route details: \"$route\", $gwpt turnpoints" # route file name
		fi
	fi
	fi
    echo -e "\t... route saved as: ~/GPS/temp/\"$rtefile\"\n\t    (edit filename and copy to ~/GPS/waypoints/ if appropriate)\n\t    the route(s) can now be cleared from the $device\n\n\tFinished\n"
fi	# report completion

#------------------------------------option------------------------------------
#   Option 5 - Jump to the next section:
elif [ "$opt" = "Next menu" ]; then
    echo

#------------------------------------option------------------------------------
#   Option 6 - Quit this script:
elif [ "$opt" = "Quit" ]; then
    echo -e "\t... exiting\n"; exit 0
else
    clear; echo -e "\t... bad option\n"; exit 0
fi

#=================================section break==================================
## UPLOAD TRACKLOG DATA ##
echo -e "\n\t------------------------------\n\t\tUPLOAD OPTIONS\n\t------------------------------\n\n\tSelect connection for the $device\n"

#   Select upload options:
choice_of()
select opt
do
   echo -e "\t$opt selected\n"
   break
done
#   Define options for this section:
choice_of "Upload waypoints to the $device" "Upload route(s) to the $device" "Next menu" "Quit"

#------------------------------------option------------------------------------
#   Option 1 - Upload waypoints or geocaches to the device:
if [ "$opt" = "Upload waypoints to the $device" ]; then
    echo -e "\t... looking for waypoint file to upload"
#   check for newest file in upload directory:
	newupl=`ls -tr $gpu/*.gpx | tail -1`    # newest file in upload directory

#   grep waypoint or geocache details:
    if [ -f "$newupl" ]; then
		gwpt=`grep "<wpt" $newupl -c`       							  # count waypoints
		geog=`grep "<sym>Geocache" $newupl -c`							  # count geocaches
		gnme=`grep "<name>" $newupl | cut -d ">" -f 2 | cut -d "<" -f 1`  # short name
		gdsc=`grep "<desc>" $newupl | cut -d ">" -f 2 | cut -d "<" -f 1`  # description
		gfrm=`grep "Freshmap" $newupl -c` 								  # Freshmap output file
		gbab=`grep "GPSBabel" $newupl -c`								  # GPSBabel output file
	else
		clear; echo -e "\n\t> > > No waypoint or geocache file in \"~/GPS/upload\" < < <\n\n\t... check and run again\n\t... exiting\n"; exit 0
	fi

#	grep the header information:
	if [ "$gfrm" -gt "1" ]; then
#	adjust output for Freshmap ".gpx" file:
		gdte=`head -6 $newupl | grep "<time>" | cut -d ">" -f 2 | cut -d "<" -f 1`	# date
		gnme=`head -6 $newupl | grep "<name>" | cut -d ">" -f 2 | cut -d "<" -f 1`	# name
		gdsc=`head -6 $newupl | grep "<desc>" | cut -d ">" -f 2 | cut -d "<" -f 1`  # desc
		gcmt=`head -10 $newupl | grep "<cmt>"  | cut -d ">" -f 2 | cut -d "<" -f 1` # comment
	else
#	adjust output for GPSBabel ".gpx" file:
		gdte=`head -10 $newupl | grep "<time>" | cut -d ">" -f 2 | cut -d "<" -f 1`	# date
		gnme=`head -10 $newupl | grep "<name>" | cut -d ">" -f 2 | cut -d "<" -f 1`	# name
		gdsc=`head -10 $newupl | grep "<desc>" | cut -d ">" -f 2 | cut -d "<" -f 1` # desc
		gcmt=`head -10 $newupl | grep "<cmt>"  | cut -d ">" -f 2 | cut -d "<" -f 1` # comment
	fi

#   report the waypoint details:
    if [[ "$gwpt" -gt "0" && "$geog" -eq "0" ]]; then
    	echo -e "\t... found waypoint file:  \"`basename $newupl`\""
    	echo -e "\t    \"$gdsc\" contains $gwpt waypoints for:\n\t     $gcmt\n"
   		read -p "        ... Do you want to upload these waypoints? (Y/N) > "
#   report the geocache details:
    elif [ "$geog" -eq "1" ]; then
    	echo -e "\t... found geocache file:  \"`basename $newupl`\""
    	echo -e "\t    single geocache:\t\"$gnme\""
    	echo -e "\t    \"$gdsc\"\n"
    	read -p "        ... Do you want to upload this geocache? (Y/N) > "
#   report the multi-geocache details:
    elif [ "$geog" -gt "1" ]; then
    	echo -e "\t... found geocache file:  \"`basename $newupl`\""
    	echo -e "\t    this is a \"multi\" containing $geog geocaches:\n$gnme"
    	read -p "        ... Do you want to upload these geocaches? (Y/N) > "
    fi
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then 
   	echo >/dev/null
    fi
    if [ "$REPLY" = "n" ] ; then 
   	echo >/dev/null
    fi
    if [ "$GO" = "1" ]; then
#	upload the waypoint or geocache
    sleep 5 && $gpsbabel -w -i gpx -f $newupl -o garmin -F $port &&
		echo -e "\t... uploaded: \"`basename $newupl`\"\n"	
    fi
    if [ $? -ne 0 ]; then
        clear; echo -e "\n\t\t> > > Transfer failed < < <\n\tCheck the following:\n\t a. the $device connected & turned on\n\t b. the correct \"port\" is specified\n\t c. no virtual machines are running (VMs can lock the output port)\n\t... exiting\n"; exit 0
    fi

#   print wapoints and rename and move the waypoint or geocache file:
    if [[ "$gwpt" -gt "0" && "$geog" -eq "0" ]]; then       # save waypoint file   
        if [ "$gwpt" -gt "1" ]; then
#			create printable version of waypoints in the ~/GPS/print directory:
#			(the next sub-section is ignored if print waypoints script is not specified in user variables)
    		if [ -f "$prtwpt" ]; then
				sh $prtwpt &&      		# run the "print-waypoints" script
				echo -e "\t    printable version in: ~/GPS/print/ directory"; sleep 2
			fi
            mv $newupl $gpw/$wptfile	# multi-waypoint saved as: <date-time>.gpx
		    echo -e "\t... saved as: ~/GPS/waypoints/\"$wptfile\"\n"
        else
            mv $newupl $gpw/$gnme.gpx	# single waypoint saved as: <waypoint>.gpx
		    echo -e "\t... saved as: ~/GPS/waypoints/\"$gnme.gpx\"\n"
        fi
    else
        if [ "$geog" -gt "1" ]; then    # save geocache file
        	mv $newupl $gpc/$gchfile	# multi-geocache saved as: <date-time>.gpx
		    echo -e "\t... saved as: ~/GPS/geocache/\"$gchfile\"\n"
        else
        	mv $newupl $gpc/$gnme.gpx	# single geocache saved as: <geocache>.gpx
		    echo -e "\t... saved as: ~/GPS/geocache/\"$gnme.gpx\"\n"
        fi
    fi
#   upload the default waypoints file from the ~/GPS/upload directory:
#	(this sub-section is ignored if no default waypoints specified in user variables)
    if [ -f "$gpw/$dwpt" ]; then
        read -p "        ... Do you want to upload the default waypoints? (Y/N) > "
    if [ "$REPLY" = "y" ] ; then
	GO="2"
    fi
    if [ "$REPLY" = "Y" ] ; then
	GO="2"
    fi
    if [ "$REPLY" = "N" ] ; then
   	echo
    fi
    if [ "$REPLY" = "n" ] ; then
   	echo
    fi
    if [ "$GO" = "2" ]; then
		$gpsbabel -w -i gpx -f $gpw/$dwpt -o garmin -F $port &&    # upload the default waypoints
		echo -e "\t    uploaded: \"`basename $dwpt`\"\n"
    fi
    if [ $? -ne 0 ]; then
        clear; echo -e "\n\t\t> > > Transfer failed < < <\n\tCheck the following:\n\t a. the $device connected & turned on\n\t b. the correct \"port\" is specified\n\t c. no virtual machines are running (VMs can lock the output port)\n\t... exiting\n"; exit 0
    fi
    fi

#   Upload the default Geocache file from the ~/GPS/upload directory:
#	(this sub-section is ignored if no default geocache file specified in user variables)
    if [ -f "$gpc/$dgeo" ]; then
        read -p "        ... Do you want to upload the default geocaches? (Y/N) > "
    if [ "$REPLY" = "y" ] ; then
	GO="3"
    fi
    if [ "$REPLY" = "Y" ] ; then
	GO="3"
    fi
    if [ "$REPLY" = "N" ] ; then
   	echo
    fi
    if [ "$REPLY" = "n" ] ; then
   	echo
    fi
    if [ "$GO" = "3" ]; then
		$gpsbabel -w -i gpx -f $gpc/$dgeo -o garmin -F $port    # upload the default geocaches
		echo -e "\t... uploaded: \"`basename $dgeo`\"\n"
    fi
    if [ $? -ne 0 ]; then
        clear; echo -e "\n\t\t> > > Transfer failed < < <\n\tCheck the following:\n\t a. the $device connected & turned on\n\t b. the correct \"port\" is specified\n\t c. no virtual machines are running (VMs can lock the output port)\n\t... exiting\n"; exit 0
    fi
	fi
	echo -e "\n\tFinished\n"	# report completion	

#------------------------------------option------------------------------------
#   Option 2 - Upload routes to the device:
elif [ "$opt" = "Upload route(s) to the $device" ]; then
    echo -e "\t... looking for route file to upload"

####################   CONVERT TRACK FILES TO ROUTES   ##########################
#
#   this section can be used to convert track files to route files, just drop 
#   a track file in the ~/GPS/upload directory, and run the script to
#	convert and upload to the GPS device.  Sometimes the script chokes if
#	if there is a newer (non-route/track) file in the uploaded directory,
#	remove this newer file and run again or "save-as" the route/track upload
#   file to make it the newest file in the ~/GPS/upload directory
#
##################################################################################

#   check for newest file in upload directory:
	newupl=`ls -tr $gpu/*.gpx | tail -1`    # newest file in upload directory
#   grep route or track details:
    if [ -f "$newupl" ]; then
	gtrk=`grep "<trkpt" $newupl -c`     # number of trackpoints
	grpt=`grep "<rtept" $newupl -c`     # number of routepoints
	gnme=`head -10 $newupl | grep "<name>" | cut -d ">" -f 2 | cut -d "<" -f 1`	# short name
	gdsc=`head -10 $newupl | grep "<desc>" | cut -d ">" -f 2 | cut -d "<" -f 1` # description
    uplrte=route-`date +%y%m%d-%H%M`.gpx
	else
		clear; echo -e "\n\t> > > No route or track file in \"~/GPS/upload\" < < <\n\n\t... check and run again\n\t... exiting\n"; exit 0
	fi

#   report the route details:
    if [ "$grpt" -gt "1" ]; then
    	echo -e "\t... found route file:  \"`basename $newupl`\":"
    	echo -e "\t    route name:\t\t\"$gnme\""
    	echo -e "\t    description:\t\"$gdsc\""
    	echo -e "\t    total routepoints:\t$grpt"
#   report the track details:
    elif [ "$gtrk" -gt "1" ]; then        #   list track details:
    	echo -e "\t... found track file: \"`basename $newupl`\""
    	echo -e "\t    track name:\t\t\"$gnme\""
    	echo -e "\t    description:\t\"$gdsc\""
    	echo -e "\t    trackpoints:\t$gtrk"
    fi
{    
#   if the file is a track file, convert to a simplified route and upload:
	if [ "$gtrk" -gt "1" ]; then
		echo -e "\n\t... the trackfile can be simplified & converted to a route\n"
		read -p "        ... Do you want to convert this trackfile to a route? (Y/N) > "
	    if [ "$REPLY" = "y" ] ; then 
		GO="1"
	    fi
	    if [ "$REPLY" = "Y" ] ; then
		GO="1"
	    fi
	    if [ "$REPLY" = "N" ] ; then 
	   	echo >/dev/null; echo -e "\t    skipping conversion"
	    fi
	    if [ "$REPLY" = "n" ] ; then 
	   	echo >/dev/null; echo -e "\t    skipping conversion"
	    fi
	    if [ "$GO" = "1" ]; then
	    echo -e "\t    converting trackfile"
	    sleep 10 && $gpsbabel -t -i gpx -f $newupl -x transform,rte=trk -x nuketypes,tracks,waypoints -x simplify,count=50 -o gpx -F $gpu/$uplrte && sleep 1
		rpts=`grep "<rtept" $gpu/$uplrte -c` &&		# grep number of routepoints 
		echo -e "\t    \"$uplrte\" - $rpts routepoints created"; sleep 2
		fi
	else
	    mv $newupl $gpu/$uplrte					# rename the newest route file for upload
	fi

#	upload route to GPS device:
	if [ -f "$gpu/$uplrte" ]; then
#		echo
		echo -e "\n\t< < < WARNING: this may overwrite previously uploaded routes > > >\n"
		read -p "        ... Do you want to upload this route now? (Y/N) > "
	    if [ "$REPLY" = "y" ] ; then 
		GO="2"
	    fi
	    if [ "$REPLY" = "Y" ] ; then
		GO="2"
	    fi
	    if [ "$REPLY" = "N" ] ; then 
	   	echo >/dev/null; echo -e "\t    skipping upload"
	    fi
	    if [ "$REPLY" = "n" ] ; then 
	   	echo >/dev/null; echo -e "\t    skipping upload"
	    fi
	    if [ "$GO" = "2" ]; then
	    echo -e "\t    uploading route"
		$gpsbabel -r -i gpx -f $gpu/$uplrte -o garmin -F $port
#	create printable version of routepoints in the ~/GPS/print directory:
#	(the next sub-section is ignored if print waypoints script is not specified in user variables)
    	if [ -f "$prtwpt" ]; then
			sh $prtwpt &&      			# run the "print-waypoints" script
			echo -e "\t    printable version in: ~/GPS/print/ directory"; sleep 2
		fi
		grpt=`grep "<rtept" $gpu/$uplrte -c` &&	# final number of routepoints
	    mv $gpu/$uplrte $gpr/$rtefile &&		# multi-route saved as <date-time>.gpx name
		echo -e "\t... saved as: ~/GPS/route/\"$rtefile\""
		rm -f $newupl 							# delete the newest route file
		echo -e "\n\tFinished\n"	# report completion	
		fi
	fi
	if [ $? -ne 0 ]; then
	    clear; echo -e "\n\t\t> > > Transfer failed < < <\n\tCheck the following:\n\t a. the $device connected & turned on\n\t b. the correct \"port\" is specified\n\t c. no virtual machines are running (VMs can lock the output port)\n\t... exiting\n"; exit 0
	fi
}
#------------------------------------option------------------------------------
#   Option 3 - Jump to the next section:
elif [ "$opt" = "Next menu" ]; then 
    echo
    
#------------------------------------option------------------------------------
#   Option 4 - Quit this script:
elif [ "$opt" = "Quit" ]; then
    echo -e "\t... exiting\n"; exit 0
else
    clear; echo -e "\t... bad option\n"; exit 0
fi

#=================================section break================================
## LIVE TRACKING OPTIONS ##
echo -e "\n\t-------------------------------------\n\t\tLIVE TRACKING OPTIONS\n\t-------------------------------------\n\n\tSelect from the live tracking options below\n"

#   Select upload options:
choice_of()
select opt
do
   echo -e "\t$opt selected\n"
   break
done
#   Define options for this section:
choice_of "TangoGPS with live tracking" "Google Earth with live tracking" "Next menu" "Quit"

#------------------------------------option------------------------------------
#   Option 1 - TangoGPS with live tracking data from the device:
if [ "$opt" = "TangoGPS with live tracking" ]; then
{
#   Start XGPS, TangoGPS & connect with GPSD daemon:
echo -e "\t... live tracking requires the $device be set to NMEA output mode\n\t... to set the $device go to: Menu > Setup > Interface > \"NMEA OUT\""
read -p "        ... is the $device is connected & turned on? (Y/N)"
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then 
   	echo
    fi
    if [ "$REPLY" = "n" ] ; then 
   	echo
    fi
    if [ "$GO" = "1" ]; then
    gpsd -b $port; echo -e "\t... starting GPSD on $port - XGPS & TangoGPS"
    echo -e "\t    please be patient, this may take a few seconds\t"; sleep 5
    echo -e "\t    to Quit - close the XGPS & TangoGPS windows"; sleep 5; xgps & tangogps
else
    echo; killall -q gpsd >/dev/null; exit 0
fi
	sleep 1; killall -q gpsd >/dev/null; echo -e "\n\t... GPSD daemon stopped, exiting"
	echo -e "\t    don't forget to set the $device back to normal mode\n"
}
#   Option 2 - Google Earth with live tracking data from the device:
elif [ "$opt" = "Google Earth with live tracking" ]; then
{
############	GOOGLE EARTH TRACKING WORKS BUT IS STILL A BIT FLAKEY ############
#	A permanent link can be made to the "live.kml" file in Google Earth with:
#	> Add > Network Link > Name: "Garmin Live Tracking", then hit "Browse" and 
#	navigate to "$HOME/GPS/temp/live.kml".  Set the refresh rate (tab) to say
#	4 seconds and click "OK" 
#	(the "live.kml" file gets deleted by this script but recreated each time the
#	"Google Earth with live tracking" is run and the "Network Link" remains)
##################################################################################
#   Start Google Earth & connect with GPSBabel:
read -p "        ... is the $device is connected & turned on? (Y/N)"
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then 
   	echo
    fi
    if [ "$REPLY" = "n" ] ; then 
   	echo
    fi
    if [ "$GO" = "1" ]; then
	echo -e "\t... creating Google Earth live tracking file in $gpt"; sleep 2
    echo -e "\t    open this file in Google Earth to view live tracking\t"; sleep 2
    echo -e "\t    to Quit - close the Google Earth & exit"; sleep 2
	# Google Earth requires .kml output for live data (will not work with .gpx etc.)
	$gpsbabel -T -i garmin -f $port -o kml,units=s -F $gpt/live.kml & googleearth
else
    echo; exit 0
fi
	killall -q gpsbabel >/dev/null && rm -f $gpt/live.kml; sleep 1
	echo -e "\n\t... Google Earth live tracking stopped, exiting"
}
#------------------------------------option------------------------------------
#   Option 3 - Jump to the next section:
elif [ "$opt" = "Next menu" ]; then 
    echo
    
#------------------------------------option------------------------------------
#   Option 4 - Quit this script:
elif [ "$opt" = "Quit" ]; then
    echo -e "\t... exiting\n"; exit 0
else
    clear; echo -e "\t... bad option\n"; exit 0
fi

#=================================section break================================
## DEVICE TOOLS, GPSD CONTROL & FINISHING OFF ##
echo -e "\n\t--------------------------\n\t\tUTILITIES\n\t--------------------------\n\n\tSelect cleanup options for the $device\n"

#   Select cleanup options:
choice_of()
select opt
do
   echo -e "\t$opt selected\n"
   break
done

#   Define options for this section:
choice_of "Convert TangoGPs track for upload to OSM" "List the latest track files" "Stop the GPSD daemon" "Turn off the $device" "Return to the start" "Quit" 

#------------------------------------option------------------------------------
#   Option 1 - Convert TangoGPs tracklog for OpenStreetMap:
if [ "$opt" = "Convert TangoGPs track for upload to OSM" ]; then
    echo -e "\tChecking for TangoGPS data log to convert"
{
#   Find the newest tracklog file in the ~/GPS/tango directory:
newlog=`ls -tr $tango/*.log | tail -1`		# newest TangoGPS log file
tname=tango-`date +%y%m%d-%H%M`				# generic TangoGPS file name

#   Check for the newest tracklog file in the ~/GPS/tango directory:
if [ ! -e "$newlog" ]; then
    echo -e "\n\t> > > Cannot find a TangoGPS data log in \"$gpt\" < < <\n"; exit 0
else
    # convert TangoGPS log file to a ".gpx" file:
    $tangcv $newlog > $tango/$tname.gpx && 	# run the "convert2gpx.pl" script
	cp $tango/$tname.gpx $gpo/$tname.gpx &&
    echo -e "\t... log file converted and copied to \"~/GPS/osm\"\n"
    echo -e "\t    (if add the location and save to ~/GPS/tracks)\n"
fi

#   Stop the gpsd daemon:
read -p "        Do you want to stop the GPSD daemon & exit? (Y/N)"
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then
   	echo
    fi
    if [ "$REPLY" = "n" ] ; then 
   	echo
    fi
    if [ "$GO" = "1" ]; then
    	if [ $test = 1 ]; then
		    killall -q gpsd >/dev/null && echo -e "\t... GPSD daemon stopped"; echo -e "\t... exiting\n"
	    else
	        killall -q gpsd >/dev/null; echo -e "\t... exiting\n"
	    fi
    fi
#   Cleaning up - remove the temp data files & report completion:
    if -f[ ! -e "$newlog" ]; then
	echo -e "\n\tFinished\n\n"
    else
	rm -f $newlog; rm -f $gpt/garmin.gpx; rm -f $gpt/garmin-clean.gpx && echo -e "\n\tFinished\n\n"
    fi
} 
#------------------------------------option------------------------------------
#   Option 2 - List the files in the upload & download directories:
elif [ "$opt" = "List the latest track files" ]; then
	newupl=`ls -tr $gpu | tail -1`    		# newest file in upload directory
	newdnl=`ls -tr $gpt | tail -1`    		# newest file in download directory
    echo -e "\t... upload directory:   \"`basename $newupl`\"\n" 
    echo -e "\t... download directory: \"`basename $newdnl`\"\n"
#------------------------------------option------------------------------------
#   Option 2 - Stop the gpsd daemon:
elif [ "$opt" = "Stop the GPSD daemon" ]; then
    killall -q gpsd >/dev/null; echo -e "\t... GPSD daemon stopped, exiting\n"

#------------------------------------option------------------------------------
#   Option 3 - Turn off the GPS device:
elif [ "$opt" = "Turn off the $device" ]; then
    echo -e "\t... powering down now"; sleep 1
    $gpsbabel -o garmin,power_off -F $port >/dev/null 2>&1
    echo -e "\t... exiting\n"; exit 0

#------------------------------------option------------------------------------
#   Option 4 - Return to the start of this script:
elif [ "$opt" = "Return to the start" ]; then
    echo -e "\t... returning to the start\n"; bash $filename

#------------------------------------option------------------------------------
#   Option 5 - Quit this script:
elif [ "$opt" = "Quit" ]; then
    echo -e "\t... exiting\n"; exit 0
else
    clear; echo -e "\t... bad option\n"; exit 0
fi

#=====================================end======================================

