#!/bin/bash
#holux_download_0.4.1.sh
#   ------------------------------------------------------------------------
#   Dave's GPS bash script is used to download Holux M-241 GPS tracklog files
#   and saves them to date-named ".gpx" & "csv" files.  The script also manages
#   the Holux M-241 GPS and runs TangoGPS but has only been tested to run in bash
#
#   COPYRIGHT:
#   Yeah Right! - (do what you like with it) but ...
#   This code comes with ABSOLUTELY NO WARRANTY.
#   It is open source and is released under the GPL; 
#   See: http://www.gnu.org/licenses/gpl.txt for full details.
#
#   IDEAS & COMMENTS:
#   This script was developed form ideas gleaned from many sources,
#   I don't claim to be any sort of guru and I would welcome any
#   sugestions, comments or improvements to: dave at 9gerkin9@gmail.com
#
#   TESTED ON:
#   The script has been used successfully on EeePC with the following OS's
#   1.  Ubuntu 8.10 Linux, kernel 2.6.24-21-eeepc
#   2.  CrunchBang 8.10.02 Linux, kernel 2.6.27-8-eeepc-lean
#   3.  CrunchBang 9.04.01 Linux, kernel 2.6.28-16-generic
#
#   & Desktop PC with the following OS's
#   1.  Ubuntu 8.10 Linux desktop edition, kernel 2.6.28-11-generic 
#   2.  Ubuntu 9.04 Linux desktop edition, kernel 2.6.28-13-generic
#   3.  Ubuntu 9.04 Linux desktop edition, kernel 2.6.28-15-generic
#   4.  Ubuntu 9.10 Linux desktop edition, kernel 2.6.31-14-generic
#
#   HARDWARE:
#   Holux M-241 datalogger, firmware version 1.11 using USB; using
#   Jaycar Electronics Tiny Bluetooth Adaptor CAT. NO. XC4892 and USB
#
#   DEPENDENCIES:
#   1. Perl script "convert2gpx.pl" to convert TangoGPS log files 
#      avaliable from:  http://www.tangogps.org/downloads/convert2gpx.pl
#   2. This script uses latest Gpsbabel - version 1.3.6, which now includes support
#      for the Holux M-241.  It was compiled from source and available from:
#        http://www.gpsbabel.org/downloaded.html
#   3. Mtkbabel (version 0.7.1) is avaliable in most linux distro repos,
#      the latest stable version (0.8-1) was found at:
#        http://sourceforge.net/project/showfiles.php?group_id=250473
#   
#   CREDITS:
#   For all this goodness see:
#   http://www.gpsbabel.org/index.html - converts waypoints, tracks, and routes
#      between popular GPS receivers and mapping programs
#   http://www.rigacci.org/ - for MTKBabel info & available from:
#   http://kambing.ui.edu/debian/pool/main/m/mtkbabel/ -MTKBabel is a Perl program 
#      to operate GPS devices based on the MediaTek MTK chipset
#   http://gpsvisualizer.com/ - a fantastic web based maps, GPS converters etc.
#   http://www.tangogps.org/gps/cat/About - TangoGPS is an easy to use, fast and
#      lightweight mapping application for use with or without GPS
#   http://openstreetmap.org/ - OpenStreetMap is a free editable map of the whole world
#   http://www.ubuntu.com/ - community developed, linux-based operating system
#
#   Thanks to members CLUG N.Z for their suggestions.
#
filename=$HOME/bin/holux_download_0.4.1.sh
version="Verson 0.4.1 - testing - 23 Dec 2009"
#
#===================================================================================
#===================	SET YOUR USER VARIBLES IN THIS SECTION		============
#===================================================================================

# List the GPS device:
device="Holux M-241"

# Local directories files are stored:
#  (the default locations here probably wont need to be changed)
gps=$HOME/GPS
gph=$HOME/GPS/holux
gpo=$HOME/GPS/osm
gpi=$HOME/GPS/print
gpt=$HOME/GPS/temp
gpk=$HOME/GPS/tracks
gpp=$HOME/GPS/WP
bin=$HOME/bin

# Set default USB download port:
usbport=/dev/ttyUSB0	# (default: /dev/ttyUSB0)

# Set default bluetooth download port:
btport=/dev/rfcomm4		# (default: /dev/rfcomm4)

# Set radius distance (eg. 0.5K, 1M) and lat/lon cordinates
# (eg. somewhere near home location to filter tracklogs for OpenStreetMap):
rdllc="distance=1K,lat=-43.580,lon=172.566"

# Location of polygon filter file:
poly=$HOME/bin/polygon_south.txt	# (not active in script yet)

# The Holux M-241 requires serial port speed to be set to 38400 baud prior to
# downloading. This is specified in the "gpsbable.ini" file:
ini=$HOME/bin/gpsbabel.ini

# set the default location TangoGPS saves the log file when in "Tracks (3/4)" here:
# (this is the location set in the "Tracks > Configure > Trackdir Config" setting)
tango=$HOME/GPS/temp

# Script variables (used in multiple sections):
#  (the default filenames here probably wont need to be changed)
trk=holux.gpx                           		# GPSBabel track file as downloaded
data=data.bin                           		# GPSBabel binary file as downloaded
cvsfile=trkhol-`date +%y%m%d-%H%M`.csv   		# TangoGPS timestamped trackfile name
trkfile=trkhol-`date +%y%m%d-%H%M`.gpx   		# Holux timestamped trackfile name
test=`ps ax | grep gpsd | grep -v grep | wc -l`	# check if the GPSD daemon is running

#===================================================================================
#==================		DON'T EDIT BELOW THIS LINE		============
#===================================================================================

echo -e "\n   ====================================================================\n\t\t\tDave's $device script\n\t\t    $version\n\n\t\tThis script downloads tracklogs and live data\n\t\tfrom the $device with using GPSBabel & GPSD\n   ====================================================================\n"
{
# check for the necessary GPS directory(s) & if not create them:
echo -e "\tChecking for required dependencies:"; sleep 1
if [ ! -d $gph ]; then
    echo -e "\n\t> > > Required GPS download directories missing in $HOME < < <\n"
    read -p "        ... shall I create the download directories now? (Y/N) > "
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then 
	echo -e "\t... please specify download directories and run again... exiting\n"; exit 0
    fi
    if [ "$REPLY" = "n" ] ; then 
	echo -e "\t... please specify download directories and run again... exiting\n"; exit 0
    fi
    if [ "$GO" = "1" ]; then
	echo -e "\t... creating the required directories"
	directories="$gps $gph $gpo $gpi $gpt $gpk $gpp $bin"
	for d in $directories
	do
		if test -d ${d}
		then
		    echo >> /dev/null
		else 
			echo -e "\t${d} not found ..."; sleep 1
			mkdir ${d}
			echo -e "\t${d} created"
		fi
	done
	fi
else
	echo -e "\t... GPS download directories are present"
fi    
}
# check for required programs needed to run this script:
# (if a required program is not present then print error and exit)
gpsbabel=`which gpsbabel`
tangcv=$bin/convert2gpx.pl
gpsd=`which gpsd`
# test to see if GPSBabel is installed:
if [ -z $gpsbabel ]; then
    echo -e "\n\t> > > Cannot find the required program \"gpsbabel\" < < <\n\n\t... version 1.3.6 can be found at:\n\t... http://www.gspbabel.org, or install it with:\n\t... \"sudo apt-get install gpsbabel\" and run again\n"; exit 0
# test to see if  the "convert2gpx.pl" script is present:
elif [ ! -e "$tangcv" ]; then
    echo -e "\n\t> > > Cannot find the required program \"convert2gpx.pl\" < < <\n\n\t... The latest version can be found at:\n\t... http://www.tangogps.org/downloads/convert2gpx.pl\n\t... download and copy to: $bin and run me again\n"; exit 0
# test to see if gpsd (gps daemon) is installed:
elif [ -z $gpsd ]; then
    echo -e "\n\t> > > Cannot find the required program \"gpsd\" < < <\n\n\t> > > The latest version can be installed with: < < <\n\t> > > \"sudo apt-get install gpsd\" & retry < < <\n"; exit 0
fi; echo -e "\t... all required programs are present\n"

#=================================section break=====================================

## SET THE COMM PORT ##
echo -e "\n\t--------------------------------\n\t\tCONNECT OPTIONS\n\t--------------------------------\n\n\tSelect connection for the $device\n"

# select connection:
choice_of()
select opt
do
   echo -e "\t$opt selected"
   break
done

# define options for this section:
choice_of "USB connection" "Bluetooth connection" "Next menu" "Quit"

#------------------------------------option------------------------------------
# Option 1 - Connect via USB port:
if [ "$opt" = "USB connection" ]; then
    echo -e "\t... check the $device is connected to USB & active\n"; port=$usbport

#------------------------------------option------------------------------------
# Option 2 - Connect via Bluetooth:
elif [ "$opt" = "Bluetooth connection" ]; then
    echo -e "\t... check the $device is turned on and the\n\t... bluetooth adapter is plugged in & active\n"; port=$btport

#------------------------------------option------------------------------------
# Option 3 - Jump to the next section:
elif [ "$opt" = "Next menu" ]; then
    echo

#------------------------------------option------------------------------------
# Option 4 - Quit this script, remove any data files, stop the GPSD daemon:
elif [ "$opt" = "Quit" ]; then
    if [ ! -e "$HOME/$data" ]; then
    	echo
    else
        rm -f $HOME/$data; rm -f $HOME/$trk			# delete the temp files
	fi
	# check if the GPSD daemon is running and if so stop it:
	if [ $test = 1 ]; then
    	killall -q gpsd >/dev/null && echo -e "\t... GPSD daemon stopped\n\t... exiting\n"; exit 0
	else
    	echo -e "\t... exiting\n"; exit 0
	fi
else
    clear; echo -e "\t... bad option\n"; exit 0
fi

#=================================section break=====================================
## DOWNLOAD TRACKLOG DATA ##
echo -e "\n\t--------------------------------\n\t\tDOWNLOAD OPTIONS\n\t--------------------------------\n\n\tSelect download options for the $device\n"

# select download options:
choice_of()
select opt
do
   echo -e "\t$opt selected\n"
   break
done

# define options for this section:
choice_of "Download the tracklog from the $device" "Download the current position" "Live tracking" "Next menu" "Quit"

#------------------------------------option------------------------------------
# Option 1 - Download the tracklog data:
if [ "$opt" = "Download the tracklog from the $device" ]; then
	# check if the GPSD daemon is running and if so stop it:
	if [ $test = 1 ]; then
	    echo -e "\t... please be patient, this may take a while"
		echo -e "\t... stopping the GPSD daemon briefly"; killall -q gpsd >/dev/null; sleep 1
	else
	    echo -e "\t... please be patient, this may take a while\n"; sleep 1
	fi &&
{
	# download the tracklog from the GPS & renames it to the correct directory:
	# (disable the "erase" option below if not clearing the device at the same time)
    read -p "	The \"erase\" option is set, continue? (Y/N) > "
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then
	GO="2"
   	echo -e "\t... skipping \"erase\", use: Option 3. below"
    fi
    if [ "$REPLY" = "n" ] ; then
	GO="2"
   	echo -e "\t... skipping \"erase\", use: Option 3. below"
    fi
    if [ "$GO" = "1" ]; then
	sleep 10 && $gpsbabel -t -i m241,erase -f $port -o gpx -F $HOME/$trk
	fi
    if [ "$GO" = "2" ]; then
	sleep 10 && $gpsbabel -t -i m241 -f $port -o gpx -F $HOME/$trk
	fi
} &&
	if [ ! -e "$HOME/$data" ]; then
		# no files downloaded to /home
	    echo -e "\n\t> > > Tracklog failed to download < < <\n\t... is the $device connected & turned on?\n\t... is waypoint logging turned on?\n"; exit 0
	else
	    mv $HOME/$trk $gpt/$trkfile &&                      # move to timestamped trackfile name
	    mv $HOME/$data $gph/gpsdata-`date +%y%m%d-%H%M`.bin  # move to timestamped bin-trackfile name
	    echo -e "\t... track downloaded as: \"~/GPS/temp/$trkfile\"\n\t... (the filename - location will need to be edited by hand)\n"
	fi

#################  STILL TO ENABLE POLYGON FILTER ##########
#
#   Filter data using ggsbabel to clean up the downloaded tracklog data:
#       a. create waypoints from tracks
#       b. remove all tracks
#       c. radius filter around home location (set radius distance & co-ords. below)
#       d. removed duplicates by location & shortname
#       e. create tracks from waypoints
#       f. remove all waypoints
#       g. appended to one another to form a single track

    $gpsbabel -t -i gpx -f $gpt/$trkfile -x transform,wpt=trk -x nuketypes,tracks -x radius,$rdllc,exclude,nosort -x transform,trk=wpt -x nuketypes,waypoints -x track,pack,split=30m,title=trkhol -o gpx -F $gpt/holux.gpx &&

    # change timestamp to 00:00 UTC:
    sed 's/T[0-9][0-9]:[0-9][0-9]:[0-9][0-9]Z/T00:00:00Z/g' $gpt/holux.gpx > $gpt/holux-clean.gpx &&

	# copy cleaned tracklog data to /temp directory:
	$gpsbabel -i gpx -f $gpt/holux-clean.gpx -o gpx -F $gpt/trkcln-`date +%y%m%d-%H%M`.gpx &&
{
	# find the newest tracklog file in the ~/GPS/temp directory:
	newest=`ls -tr $gpt/trkcln-*.gpx | tail -1`		# newest file in the temp directory
	osmfile=osmhol-`date +%y%m%d-%H%M`.gpx			# OpenStreetMap road track file name
if [ ! -e "$newest" ]; then
    echo -e "\t> > >  Cannot find the \"trkcln-*.gpx\" file in \"$gpt\" < < <\n"
else
    read -p "	Is this track going to be uploaded to OpenStreetMap? (Y/N) > "
    if [ "$REPLY" = "y" ] ; then 
	GO="3"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="3"
    fi
    if [ "$REPLY" = "N" ] ; then
   	echo
    fi
    if [ "$REPLY" = "n" ] ; then 
   	echo
    fi
    if [ "$GO" = "3" ]; then
    # copy to the OpenStreetMap directory
	cp $newest $gpo/$osmfile && echo -e "\t... OSM track saved as: \"~/GPS/osm/$osmfile\"\n"
    fi
fi
} &&
    # cleaning up - remove the temp data files & report completion:
    if [ ! -e "$newest" ]; then
	    echo
    else
	    rm $newest; rm -f $gpt/holux.gpx; rm -f $gpt/holux-clean.gpx
    fi
	# restart the GPSD daemon if it was previously running:
	if [ $test = 1 ]
	then
		echo -e "\t... restarting the GPSD daemon\n"
	    gpsd -b $port; sleep 1 && echo -e "\tFinished\n"
	else
		echo -e "\tFinished\n"
	fi

#------------------------------------option------------------------------------
# Option 2 - Grab current position & convert to waypoint:
elif [ "$opt" = "Download the current position" ]; then
	# check if the GPSD daemon is running and if so stop it:
	if [ $test = 1 ]; then
	    echo -e "\t... please be patient, this may take a while"
		echo -e "\t... stopping the GPSD daemon briefly"; killall -q gpsd >/dev/null; sleep 1
	else
	    echo -e "\t... please be patient, this may take a while"
	fi &&
{
    # download the tracklog from the GPS & renames it to the correct directory:
    sleep 10 && $gpsbabel -t -i m241 -f $port -o gpx -F $HOME/$trk &&
	if [ ! -e "$HOME/$data" ]; then
	    echo -e "\n\t> > > Tracklog failed to download < < <\n\t... is the $device connected to $port & turned on?\n\t... is waypoint logging turned on?\n"; exit 0	# no data file downloaded to /home
	else
    	mv $HOME/$trk $gpt/$trkfile		# move to timestamped trackfile name
	fi
if [ ! -e "$gpt/$trkfile" ]; then
    echo -e "\n\t> > > Tracklog failed to download < < <\n\t... is the $device connected to $port & turned on?\n"; exit 0	# no track file in temp
else
    # convert "$trkfile" to .csv format and copy output file to the ~/GPS/temp folder
	$gpsbabel -i gpx -f $gpt/$trkfile -x transform -o csv -F $gpt/$cvsfile
    # grab last 10 lines of tracklog (csv) to create a "current_position.gpx" waypoint "blop"
    # for TangoGPS, load with: TangoGPS > Track > Load > "current_position.csv"
    # create 2 separate ".csv" files from the downloaded :$gpt/$cvsfile"
    tail -n 10 $gpt/$cvsfile > $tango/current_position.csv; tail -n 1 $gpt/$cvsfile > $gpt/1posn.csv
	echo -e "\t... current position saved as:\n\t     1. ~/GPS/tracks/current_position.csv"
	
    # create a "current_position.gpx" waypoint for Freshmap, grab last line of tracklog "csvfile":
    # and convert output to single waypoint file:
	$gpsbabel -i csv -f $gpt/1posn.csv -x transform,wpt=trk -o gpx -F $gpp/position.gpx
    # position file names:
	wptf=WP`date +%y%m%d-%H%M`.gpx   # waypoint - long format timestamp
	wptn=WP`date +%H%M`             # waypoint - short format timestamp
	wpty=current_position.gpx       # current position .gpx file
    # rename the <name>, <cmt> & <desc>  fields with timestamps/comments etc:
	sed 's/<name>[TP0123456789]*/<name>'$wptn'/' $gpp/position.gpx | 
	sed 's/<cmt>[TP0123456789]*/<cmt>Position as at: '`date +%H%M`' hrs '`date +%d.%m.%Y`'/' | 
	sed 's/<desc>[TP0123456789]*/<desc>'$wptn'/' > $gpp/$wptf &&
	
    # copy waypoints and clean up - remove the temp data files & report completion:
	cp $gpp/$wptf $gps/$wpty && echo -e "\t     2. ~/GPS/$wpty\n\t     3. ~/GPS/WP/$wptf\n"
	rm -f $gpp/position.*; rm -f $gpt/1posn.csv; rm -f $HOME/$data &&
	
    # restart the GPSD daemon only if it was previously running:
	if [ $test = 1 ]
	# exit in this section to prevent gpsd getting killed when script quits
	then
		echo -e "\t... restarting the GPSD daemon\n"
			    gpsd -b $port; sleep 1 && echo -e "\tFinished\n\n"; exit 0	# report completion	    
	else
        # exit here to prevent gpsd getting killed when script quits
		echo -e "\tFinished\n"; exit 0        # report completion
	fi
fi
}
#------------------------------------option------------------------------------
# Option 3 - Live tracking data from the device:
elif [ "$opt" = "Live tracking" ]; then
	echo -e "\t... check that the $device turned on, bluetooth is connected & active\n"

echo -e "\t 1   =  Start the GPSD daemon & XGPS\n\t 2   =  Start GPSD and TangoGPS Map Viewer\n\t 3   =  Start GPSD and Viking Map Editor\n\t 4   =  Stop the GPSD daemon\n\tN/n  =  Next menu\n\tQ/q  =  Quit\n"

{
# start the GPSD daemon:
read -p "	... select from the options above: > "
	# start the GPSD daemon & XGPS:
    if [ "$REPLY" = "1" ] ; then
		gpsd -b $port; echo -e "\t... starting GPSD on $port"; sleep 1
	    echo -e "\t... please be patient, this may take a few seconds\t"; sleep 5
	    echo -e "\t... to Quit - close the XGPS window"; sleep 5; xgps
    fi
	# start the GPSD daemon, XGPS & TangoGPS:
    if [ "$REPLY" = "2" ] ; then
	    echo -e "\t... starting the TangoGPS Map Viewer"; sleep 1
	    echo -e "\t... please be patient, this may take a few seconds\t"; sleep 1
		gpsd -b $port; echo -e "\t... starting GPSD on $port"; sleep 5
	    echo -e "\t... to Quit - close the TangoGPS window"; sleep 3; xgps & tangogps
	fi
	# start the GPSD daemon, XGPS & Viking Map Viewer:
    if [ "$REPLY" = "3" ] ; then
	    echo -e "\t... starting the Viking Map Editor"; sleep 1
	    echo -e "\t... please be patient, this may take a few seconds\t"; sleep 1
		gpsd -b $port; echo -e "\t... starting GPSD on $port"; sleep 5
	    echo -e "\t... to view live track: open or create a Viking - GPS Layer\n\t... for live tracking in the Viking Map Editor"; sleep 1
	    echo -e "\t... to Quit - close the Viking window"; sleep 3; xgps & viking
    fi
	# stop the GPSD daemon:
    if [ "$REPLY" = "4" ] ; then
		# check if the GPSD daemon is running and if so stop it:
		if [ $test = 1 ]; then 
			killall -q gpsd >/dev/null && echo -e "\t... GPSD daemon stopped\n"
		else
			echo -e "\t... GPSD daemon was not running\n"
		fi
	fi
    if [ "$REPLY" = "N" ] ; then 
   	echo
    fi
    if [ "$REPLY" = "q" ] ; then 
		if [ $test = 1 ]; then
			killall -q gpsd >/dev/null; sleep 1; echo -e "\n\tQuit selected\n\t... GPSD daemon stopped\n\t... exiting\n"; exit 0
		else
		    killall -q gpsd >/dev/null; echo -e "\n\tQuit selected\n\t... exiting\n"; exit 0
	    fi
	fi
    if [ "$REPLY" = "Q" ] ; then 
    # check if the GPSD daemon is running and if so stop it:
		if [ $test = 1 ]; then 
			killall -q gpsd >/dev/null; sleep 1; echo -e "\t... GPSD daemon stopped\n\t... exiting\n"; exit 0
		else
		    killall -q gpsd >/dev/null; echo -e "\n\tQuit selected\n\t... exiting\n"; exit 0
		fi
	fi
}
#------------------------------------option------------------------------------
# Option 4 - Jump to the next section:
elif [ "$opt" = "Next menu" ]; then
    echo

#------------------------------------option------------------------------------
# Option 5 - Quit this script, remove any data files, stop the GPSD daemon:
elif [ "$opt" = "Quit" ]; then
    if [ ! -e "$HOME/$data" ]; then
    	echo
    else
        rm -f $HOME/$data; rm -f $HOME/$trk			# delete the temp files
	fi
	# check if the GPSD daemon is running and if so stop it:
	if [ $test = 1 ]; then
    	killall -q gpsd >/dev/null && echo -e "\t... GPSD daemon stopped\n\t... exiting\n"; exit 0
	else
    	echo -e "\t... exiting\n"; exit 0
	fi
else
    clear; echo -e "\t... bad option\n"; exit 0
fi

#=================================section break=====================================
## CONVERT TRACKS, TOOLS, GPSD CONTROL & FINISHING OFF ##
echo
echo "	  	----------------------------------"
echo "	  	   CONVERT TRACKS & CLEANING UP"
echo "	  	----------------------------------"
echo
echo "   Select cleanup options for the $device"
echo

# select final options:
choice_of()
{
select opt
do
    echo
    echo "      $opt selected"
    echo
    break
done
}

# define options for this section:
choice_of "Convert TangoGPs track for upload to OSM" "Stop the GPSD daemon" "Clear the $device" "Force reset the $device" "Return to the start" "Quit" 

#------------------------------------option------------------------------------
# Option 1 - Convert TangoGPs tracklog for OpenStreetMap:
if [ "$opt" = "Convert TangoGPs track for upload to OSM" ]; then
    echo -e "\t... checking for TangoGPS data log to convert"
{
# so find the newest TangoGPS tracklog file in the  directory:
newlog=`ls -tr $tango/*.log | tail -1`		# newest TangoGPS log file
tname=tango-`date +%y%m%d-%H%M`				# generic TangoGPS file name

# check for the newest tracklog file in the ~/GPS/tango directory:
if [ ! -e "$newlog" ]; then
    echo -e "\n\t> > > Cannot find a TangoGPS data log in \"$gpk\" < < <\n"; exit 0
else
    # convert TangoGPS log file to a ".gpx" file:
    $tangcv $newlog > $tango/$tname.gpx && 	# run the "convert2gpx.pl" script
	cp $tango/$tname.gpx $gpo/$tname.gpx &&
    echo -e "\t... log file converted and copied to \"~/GPS/osm\"\n"
    echo -e "\t... (if add the location and save to ~/GPS/tracks)\n"    
fi

# stop the gpsd daemon:
read -p "        Do you want to stop the GPSD daemon & exit? (Y/N) > "
    if [ "$REPLY" = "y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "Y" ] ; then 
	GO="1"
    fi
    if [ "$REPLY" = "N" ] ; then 
   	echo
    fi
    if [ "$REPLY" = "n" ] ; then 
   	echo
    fi
    if [ "$GO" = "1" ]; then
    	if [ $test = 1 ]; then
		    killall -q gpsd >/dev/null && echo -e "\t... GPSD daemon stopped"; echo -e "\t... exiting\n"
	    else
	        killall -q gpsd >/dev/null; echo -e "\t... exiting\n"
	    fi
    fi
#   Cleaning up - remove the temp data files & report completion:
    if [ ! -e "$newlog" ]; then
	echo -e "\n\tFinished\n\n"
    else
	rm -f $newlog && echo -e "\n\tFinished\n\n"
    fi
}
#------------------------------------option------------------------------------
# Option 2 - Stop the gpsd daemon:
elif [ "$opt" = "Stop the GPSD daemon" ]; then
	# check if the GPSD daemon is running and if so stop it:
	if [ $test = 1 ]; then 
		killall -q gpsd >/dev/null && echo -e "\t... GPSD daemon stopped\n\t... exiting\n"
	else
		echo -e "\t... GPSD daemon was not running\n\t... exiting\n"; exit 0
	fi

#------------------------------------option------------------------------------
# Option 3 - Clear data from the GPS device:
elif [ "$opt" = "Clear the $device" ]; then
    echo -e "\t... please be patient, this may take a while"
    sleep 10 && $gpsbabel -t -i m241,erase -f $port; rm -f $HOME/$data && echo -e "\t... $device datalog cleared\n\t... exiting\n"; exit 0	# report completion

#------------------------------------option------------------------------------
# Option 4 - Force reset the GPS device:  
# (NOTE:    MTKBabel is only required for the "force-reset" option here)
elif [ "$opt" = "Force reset the $device" ]; then
    # test to see if mtksbabel is installed
    mtkbabel=`which mtkbabel`
    if [ -z $mtkbabel ]; then
        echo -e "\t> > > Cannot find the required program \"mtkbabel\" < < <\n\n\t> > > version 0.7 or higher.  It can be found at: < < <\n\t> > > https://sourceforge.net/projects/mtkbabel < < <\n\t> > > or install it with: \"sudo apt-get install mtkbabel\" & retry < < <\n"; exit 0
    else
        echo -e "\t... please be patient, this may take a while\n"
        $mtkbabel -s 38400 -p $port -r 1:50:0 -m stop -l on -R && echo -e "\n\t... $device reset & cleared\n\t... exiting\n"; exit 0	# report completion
    fi

#------------------------------------option------------------------------------
# Option 5 - Return to the start of this script:
elif [ "$opt" = "Return to the start" ]; then
    echo -e "\t... returning to the start\n"; bash $filename

#------------------------------------option------------------------------------
# Option 6 - Quit this script, remove any data files, stop the GPSD daemon:
elif [ "$opt" = "Quit" ]; then
    echo -e "\t... cleaning up"
    # delete the tempoary files: 
    if [ ! -e "$HOME/$data" ]; then
        exit 0
    else
		echo -e "\t... removing temporary files"
        rm -f $HOME/$data; rm -f $HOME/$trk; exit 0
	fi & sleep 1
	# check if the GPSD daemon is running and if so stop it:
	if [ $test = 1 ]; then
		killall -q gpsd >/dev/null && echo -e "\t... GPSD daemon stopped"; echo -e "\t... exiting\n"
	else
	    killall -q gpsd >/dev/null; echo -e "\t... exiting\n"
	fi
else
    clear; echo -e "\t... bad option\n"; exit 0
fi

#===================================end============================================

