#!/bin/bash
#print_waypoints_0.18.sh
#------------------------------------------------------------------------------
#   Dave's GPS bash script is used to reformat waypoint ".gpx" files to a 
#   print friendly version by replacing or removing unnecessary html tags.
#
#   COPYRIGHT:
#   Yeah Right! - (do what you like with it) but ...
#   This code comes with ABSOLUTELY NO WARRANTY.
#   It is open source and is released under the GPL
#   See: http://www.gnu.org/licenses/gpl.txt for full details.
#
#   IDEAS & COMMENTS:
#   This script was developed form ideas gleaned from many sources,
#   I don't claim to be any sort of guru and I would welcome any
#   sugestions, comments or improvements to: dave at 9gerkin9@gmail.com
#
#   TESTED ON:
#   The script has been used successfully on EeePC with the following OS's
#   1.  CrunchBang 9.04.01 Linux, kernel 2.6.28-16-generic
#
#   & Desktop PC with the following OS's
#   1.  Ubuntu 9.10 Linux desktop edition, kernel 2.6.31-15-generic
#
#   DEPENDENCIES:
#   1. 
#
#   CREDITS:
#
#
filename=$HOME/bin/print_waypoints_0.18.sh
version="Verson 0.18 - testing - 21 Jan 2010"
#
#==============================================================================
#==================	  SET YOUR USER VARIBLES IN THIS SECTION	===============
#==============================================================================

#   Local folders where files are stored:
#  (the default locations here probably wont need to be changed)
gps=$HOME/GPS
gpc=$HOME/GPS/gcmtache
gpo=$HOME/GPS/osm
gpi=$HOME/GPS/print
gpr=$HOME/GPS/routes
gpn=$HOME/GPS/tango
gpt=$HOME/GPS/temp
gpk=$HOME/GPS/tracks
gpu=$HOME/GPS/upload
gpw=$HOME/GPS/waypoints
gpp=$HOME/GPS/WP
bin=$HOME/bin

prtwpt=print_waypoints-`date +%H%M`			# final printable output file
prtrte=print_route-`date +%H%M`				# final printable output file

#==============================================================================
#===================	DON'T EDIT BELOW THIS LINE		=======================
#==============================================================================

#   check for newest file in upload directory:
	newupl=`ls -tr $gpu/*.gpx | tail -1`    # newest file in upload directory
	header=$gpi/header.txt
	outfile=$gpi/outfile.txt

#echo "	... running: \"`basename $filename`\"\n" ###TESTING### 
#rm -rf $outfile; rm -rf $header && ###TESTING###

#   grep waypoint or route details:
if [ -f "$newupl" ]; then
	gwpt=`grep '<wpt' $newupl -c`       							  # count waypoints
#	grpt=`grep '<rtept' $newupl -c`       					          # count routepoints
    grpt=`sed 's/<rtept/<rtept\n/g' $newupl | grep -c '<rtept'`		  # count routepoints
	gnme=`grep '<name>' $newupl | cut -d ">" -f 2 | cut -d "<" -f 1`  # short name
	gdsc=`grep '<desc>' $newupl | cut -d ">" -f 2 | cut -d "<" -f 1`  # description
	gfrm=`grep "Freshmap" $newupl -c` 								  # Freshmap output file
	gbab=`grep 'GPSBabel' $newupl -c`								  # GPSBabel output file
	gclm=`grep 'CloudMade' $newupl -c`								  # Cloudemade download file
else
	echo -e "\n\t> > > No waypoint or route file in \"~/GPS/print\" < < <\n"; exit 0
fi
#
#	This section is used to convert routes (gpx) created with
#	Freshmap to a printable route and directions list:
if [ "$gfrm" -gt "0" ]; then
#    echo "FM-1" ###TESTING###
	echo "\t... printing Freshmap waypoint or route file"
#	adjust output for Freshmap ".gpx" file:
	cat $newupl | sed -e '1,2d' -e '/Freshmap/d' -e '/extensions/d' -e 's|<wpt|Waypoint:\t|' -e 's|">|"|' -e 's|<rtept|Routepoint:\t|' -e 's/">/"/' -e 's|<ele>|   Elevation:\t|' -e 's|</ele>| m|' -e 's|<name>|   Name:\t|' -e 's|</name>||' -e 's|<cmt>|   Comment:\t|' -e 's|</cmt>||' -e '/<desc>/d' -e '/<time>/d' -e 's|</sym>||' -e 's|<type>|   Symbol:\t|' -e 's|<sym>|   Symbol:\t|' -e 's|</type>||' -e 's|</wpt>||' -e 's|</rtept>||' -e '/gpx>/d' -e '/root>/d' -e '/rte>/d' -e '$d' -e 's|&quot;|"|' -e 's|&amp;|&|' -e 's|&apos;|`|' > $outfile
#	grep the file details:
#	gdte=`head -8 $newupl | grep '<time>' | cut -d ">" -f 2 | cut -d "<" -f 1`	# date
	gdte=`stat -c %y $newupl | sed 's/^\([0-9\-]*\).*/\1/'`						# date
	gnme=`head -8 $newupl | grep '<name>' | cut -d ">" -f 2 | cut -d "<" -f 1`	# name
	gdsc=`head -8 $newupl | grep '<desc>' | cut -d ">" -f 2 | cut -d "<" -f 1`  # desc
	gcmt=`head -8 $newupl | grep '<cmt>'  | cut -d ">" -f 2 | cut -d "<" -f 1`  # comment
	
#   create a header file route details grep'd above:
if [ "$gwpt" -gt "1" ]; then
	echo "=============== WAYPOINTS =============" >> $header
	echo "Created:	$gdte" >> $header
	echo "Printed:	`date +%d-%b-%Y`" >> $header
	echo "Filename:	\"`basename $newupl`\"" >> $header
	echo "Description:	$gdsc" >> $header
	echo "Comment:	$gcmt" >> $header
	echo "Waypoints:	$gwpt" >> $header
	echo "=====================================" >> $header
	echo "" >> $header
elif [ "$grpt" -gt "1" ]; then
	echo "============ NAVIGATION ROUTE =========" >> $header
	echo "Created:	$gdte" >> $header
	echo "Printed:	`date +%d-%b-%Y`" >> $header
	echo "Filename:	\"`basename $newupl`\"" >> $header
	echo "Routename:	$gnme" >> $header
	echo "Description:	$gdsc" >> $header
	echo "Comment:	$gcmt" >> $header
	echo "Routepoints:	$grpt" >> $header
	echo "Distance:	 km" >> $header
	echo "=====================================" >> $header
	echo "" >> $header
fi
#
#	This section is used to convert routes (gpx) created with
#	GPSBabel to a printable route and directions list:
elif [ "$gbab" -gt "0" ]; then
#    echo "BL-1" ###TESTING###
	echo "\t... printing GPSBabel waypoint or route file"
#	adjust output for GPSBabel ".gpx" file:
	cat $newupl | sed -e '1,9d' -e '/bounds/d' -e '/ACTIVE LOG/d' -e 's|<wpt |Waypoint:\t|' -e 's|  <rtept |Routepoint:\t|' -e 's/">/"/' -e 's|<ele>| Elevation:\t|' -e 's|</ele>| m|' -e 's|<name>| Name:\t|' -e 's|</name>||' -e 's|<cmt>| Comment:\t|' -e 's|</cmt>||' -e '/<desc>/d' -e '/<time>/d' -e 's|<sym>| Symbol:\t|' -e 's|</sym>||' -e 's|</wpt>||' -e 's|</rtept>||' -e '/gpx>/d' -e '/rte>/d' > $outfile
#	grep the file details:
#	gdte=`head -10 $newupl | grep '<time>' | cut -d ">" -f 2 | cut -d "<" -f 1`	# date
	gdte=`stat -c %y $newupl | sed 's/^\([0-9\-]*\).*/\1/'`						# date
	gnme=`head -10 $newupl | grep '<name>' | cut -d ">" -f 2 | cut -d "<" -f 1`	# name
	gdsc=`head -10 $newupl | grep '<desc>' | cut -d ">" -f 2 | cut -d "<" -f 1` # desc
	gcmt=`head -10 $newupl | grep '<cmt>'  | cut -d ">" -f 2 | cut -d "<" -f 1` # comment
	
#   create a header file route details grep'd above:
if [ "$gwpt" -gt "1" ]; then
	echo "=============== WAYPOINTS =============" >> $header
	echo "Created:	$gdte" >> $header
	echo "Printed:	`date +%d-%b-%Y`" >> $header
	echo "Filename:	\"`basename $newupl`\"" >> $header
	echo "Description:	$gdsc" >> $header
	echo "Comment:	$gcmt" >> $header
	echo "Waypoints:	$gwpt" >> $header
	echo "=====================================" >> $header
	echo "" >> $header
elif [ "$grpt" -gt "1" ]; then
	echo "============ NAVIGATION ROUTE =========" >> $header
	echo "Created:	$gdte" >> $header
	echo "Printed:	`date +%d-%b-%Y`" >> $header
	echo "Filename:	\"`basename $newupl`\"" >> $header
	echo "Routename:	$gnme" >> $header
	echo "Description:	$gdsc" >> $header
	echo "Comment:	$gcmt" >> $header
	echo "Routepoints:	$grpt" >> $header
	echo "Distance:	 km" >> $header
	echo "=====================================" >> $header
	echo "" >> $header
fi
#
################### TRY AND GET ROUTEPOINT NUMBERS IN PRINTOUT?? ###########
#
#	This section is used to convert route (gpx) downloaded from
#	"http://cloudmade.com" to a printable route and directions list:
elif [ "$gclm" -gt "0" ]; then
#    echo "CM-1" ###TESTING###
	echo "\t... printing CloudMade route file"
#	use sed to strip/replace html tags from CloudMade ".gpx" file:
    cat $newupl | sed -e 's|<?xml|DELETE |' -e 's|</extensions>|\nDELETE|' -e 's|<name>|\n<name>downloaded from |' -e 's|<email|\nDELETE |' -e 's|href="|\n<desc>see: |' -e 's|"></link>|</desc>|' -e 's|</author>|\nDELETE |' -e 's|copyright>|\n|' -e 's|+0000</time></metadata><extensions>|</time>|' -e 's|<distance>|\n<distance>|' -e 's|</distance><time>|</distance>\nDELETE |' -e 's/<start>/\n<start>/' -e 's|<end>|\n<end>|' -e 's|</extensions>|\nDELETE |' -e 's|<rte>||' -e 's|<rtept |\n\nRoutepoint:\t|g' -e 's|</rtept>||' -e 's|><desc>|\n  Directions:\t|g'  -e 's|0.0</turn-angle>|\n\tand travel straight ahead|g' -e 's|<direction>|\n  Turn ahead:\t|g'  -e 's|</direction><azimuth>| - bearing: |g' -e 's|.[0-9]</azimuth>|°\nDELETE |g' -e 's|</extensions></rtept>|\nDELETE |g' -e 's|</desc><|\nDELETE|g' -e 's|</rte></gpx>|\n|' > $outfile &&
#	grep the file details:
	gdte=`head -10 $outfile | grep '<time>' | cut -d ">" -f 2 | cut -d "<" -f 1`	 # date
	gnme=`head -10 $outfile | grep '<name>' | cut -d ">" -f 2 | cut -d "<" -f 1`	 # name
	gdsc=`head -10 $outfile | grep '<desc>' | cut -d ">" -f 2 | cut -d "<" -f 1`	 # desc
	gcmt=`head -10 $outfile | grep '<cmt>' | cut -d ">" -f 2 | cut -d "<" -f 1`		 # comment
	star=`head -10 $outfile | grep '<start>' | cut -d ">" -f 2 | cut -d "<" -f 1`	 # start route
	endr=`head -10 $outfile | grep '<end>' | cut -d ">" -f 2 | cut -d "<" -f 1`		 # end route
	dist=`head -10 $outfile | grep '<distance>' | cut -d ">" -f 2 | cut -d "<" -f 1` # route length (m)

#   create a header file from the route details grep'd above:
	echo "============ NAVIGATION ROUTE ===========" >> $header
	echo "Created:	$gdte" >> $header
	echo "Printed:	`date +%d-%b-%Y`" >> $header
	echo "Filename:	\"`basename $newupl`\"" >> $header
	echo "Routename:	$gnme" >> $header
	echo "Description:	$gdsc" >> $header
	echo "From:		$star" >> $header
	echo "To:		$endr" >> $header
	echo "Routepoints:	$grpt" >> $header
	echo "Distance:	$(( $dist/1000 )) km" >> $header
	echo "=====================================" >> $header
fi	

#	clean up the resulting output and cat the header and output file
#
#	adjust output for Freshmap ".gpx" file:
if [ "$gfrm" -gt "0" ]; then
#	echo "FM-2" ###TESTING###
#	remove unwanted lines from the route, join the header and output:
	cd $gpi; cat $outfile | sed -e '1d' -e '$d' -e '/DELETE/d' > $outfile | cat $header $outfile > $prtrte
#
#	adjust output for GPSBabel ".gpx" file:
elif [ "$gbab" -gt "0" ]; then
#	echo "BL-2" ###TESTING###
#	remove unwanted lines from the route, join the header and output:
	cd $gpi; cat $outfile | sed -e '/DELETE/d' > $outfile | cat $header $outfile > $prtrte
#	echo "testing direct print option" ###TESTING###
#	cd $gpi; cat $header $outfile | sed -e '10d' -e '/DELETE/d' | lpr ###TESTING###	
#
#	adjust output for CloudMade ".gpx" file:
elif [ "$gclm" -gt "0" ]; then
#	echo "CM-2" ###TESTING###
#	remove unwanted lines from the route, join the header and output:
	cd $gpi; cat $outfile | sed -e '1,10d' -e '/DELETE/d' > $outfile | cat $header $outfile > $prtrte
fi

#   Cleaning up - remove the temp waypoint & header files & report completion:
rm -rf $outfile; rm -rf $header && ###TESTING###
#	echo -e "\n\t\"$outfile\" - created"
#	echo -e "\n\tFinished\n"

exit 0

